#include <cstdio>
#include <cstdlib>
#include <cstring>

#define SBOOL(x) ((x) ? "true" : "false")
#define BBP "0b%d%d%d%d%d%d%d%d"
#define BB(x) \
((x) & 0x01 ? 1 : 0), \
((x) & 0x02 ? 1 : 0), \
((x) & 0x04 ? 1 : 0), \
((x) & 0x08 ? 1 : 0), \
((x) & 0x10 ? 1 : 0), \
((x) & 0x20 ? 1 : 0), \
((x) & 0x40 ? 1 : 0), \
((x) & 0x80 ? 1 : 0)

typedef unsigned char u8;
typedef unsigned long u32;
typedef unsigned long long u64;

#pragma pack(push,1)
struct DTMHeader {
	u8 filetype[4];			// Unique Identifier (always "DTM"0x1A)

	u8 gameID[6];			// The Game ID
	bool bWii;				// Wii game

    u8  numControllers;		// The number of connected controllers (1-4)

    bool bFromSaveState;	// false indicates that the recording started from bootup, true for savestate
    u64 frameCount;			// Number of frames in the recording
	u64 InputCount;			// Number of input frames in recording
    u64 lagCount;			// Number of lag frames in the recording
    u64 uniqueID;			// A Unique ID comprised of: md5(time + Game ID)
    u32 numRerecords;		// Number of rerecords/'cuts' of this TAS
    u8  author[32];			// Author's name (encoded in UTF-8)
    
    u8  videoBackend[16];	// UTF-8 representation of the video backend
    u8  audioEmulator[16];	// UTF-8 representation of the audio emulator
    u8  padBackend[16];		// UTF-8 representation of the input backend

    u8  reserved[127];		// Make heading 256 bytes, just because we can
};
#pragma pack(pop)

int main(int argc, char **argv)
{
	DTMHeader head;
	FILE *f;

	memset(&head, 0, sizeof(DTMHeader));

	if (argc != 2)
	{
		printf("usage: %s movie.dtm\n", argv[0]);
		return 1;
	}

	f = fopen(argv[1], "rb");
	if (f==NULL)
	{
		fputs("could not open file\n", stderr);
		return 1;
	}

	fread(&head, 256, 1, f);

	printf("filetype:       %c%c%c0x%X\n", head.filetype[0], head.filetype[1], head.filetype[2], head.filetype[3]);
	printf("gameID:         %c%c%c%c%c%c\n", head.gameID[0], head.gameID[1], head.gameID[2], head.gameID[3], head.gameID[4], head.gameID[5]);
	printf("bWii:           %s\n", SBOOL(head.bWii));
	printf("numControllers: " BBP "\n", BB(head.numControllers));
	printf("bFromSaveState: %s\n", SBOOL(head.bFromSaveState));
	printf("frameCount:     %d\n", head.frameCount);
	printf("InputCount:     %d\n", head.InputCount);
	printf("lagCount:       %d\n", head.lagCount);
	printf("uniqueID:       %X\n", head.uniqueID);
	printf("numRerecords:   %d\n", head.numRerecords);
	printf("author:         %s\n", head.author);
	printf("videoBackend:   %s\n", head.videoBackend);
	printf("audioEmulator:  %s\n", head.audioEmulator);
	printf("padBackend:     %s\n", head.padBackend);

#ifdef _WIN32
	printf("\n");
	system("pause");
#endif

	return 0;
}
